/*
   ----------------------------------------------------------------------------
   HouseholdExensesText.prg
   Sample code from The dBASE Reports Book, second edition
   Chapter 21
   Copyright  2016, Kenneth J. Mayer
   
   Program used to generate a text file version of a report from the
   Household table.
   
   This may be used for other purposes, as needed. 
   ----------------------------------------------------------------------------
*/
// Set up the database and query objects:
local dData, qHE, rHE, fHE
dData = new database()
dData.databaseName := "DBASEREPORTS"
dData.active       := true

qHE = new query()
qHE.database       := dData
qHE.sql            := "select * from Household"
qHE.active         := true

// shortcuts
rHE = qHE.rowset
fHE = rHE.fields

// index:
rHE.indexName := "ENTRYDATE"

// set up information for "report"
nWidth = 80

// set up file object:
fOut = new file()
fOut.create( "HouseholdExpenses.txt", "W" ) // Write access

// output the headings:
// "MainTitle"
cHeading = "My Company Name"
fOut.puts( CenterText( nWidth, cHeading ) )
// "SecondaryHeading"
cHeading = "Household Expenses"
fOut.puts( CenterText( nWidth, cHeading ) )
// Date:
cHeading = cmonth(date())+" "+day(date())+", "+year(date())
fOut.puts( CenterText( nWidth, cHeading ) )
// blank line:
fOut.puts( "" )

// column headings -- tricky, because we need to get everything
// to line up ...

// The Date will take 10 characters (MM/DD/YYYY)
// The Category is defined as 24 characters
// the Amount field is 7.2 (seven digits, 2 decimals)
//    but we want it to look like: 99,999.99 -- so add
//    one more for the comma (width of 8)
// The Note field is maxed out at 254 characters,
//     but most notes are not that long ...
// We should have at least a couple of blank spaces between
// columns. However, the words for the headings are even
// trickier:
cColumnHeadings  = "Entry"
cColumnHeadings += replicate( " ", 7 ) // 10-5 (5) for the field 
                                       // and 2 for blank space
cColumnHeadings += "Category"
cColumnHeadings += replicate( " ", 19 ) // 25-8 (17) and 2 for blank

cColumnHeadings += " Amount"
cColumnHeadings += replicate( " ", 4 ) // 9-7 (2) and 2 for blank

cColumnHeadings += "Note"
// output the headings:
fOut.puts( cColumnHeadings )

// loop through the table and list the data out:
rHE.first() // first row in table
do while not rHE.endOfSet // loop to end of table

   // initialize the text variable:
   cRow = ""
   
   // EntryDate:
   cRow += fHE["EntryDate"].value
   
   // blank spaces:
   cRow += "  " // 2 spaces
   
   // Category -- normally when outputting such
   // a value, I would trim the field. However,
   // in this case, let's just use the "untrimmed" 
   // value which will include spaces on the right:
   cRow += fHE["Category"].value
  
   // blank spaces:
   cRow += "  " // 2 spaces
   
   // Amount is tricky -- we want it to line
   // up on the right, and we want a picture
   // for it so that we have commas ...
   // we will use the transform() function
   // and a picture:
   cRow += transform( fHE["Amount"].value, "99,999.99" )
   
   // blank spaces
   cRow += "  " // 2 spaces
   
   // Note -- we want to truncate this to keep it at
   // 80 characters per line. We are currently using
   // 50 characters, so the most we can use is 30:
   // The other problem is that if the value of the
   // "Note" field is null (empty), we end up wiping out 
   // the string here (string+null = null).
   cNote = new string( fHE["Note"].value.rightTrim())
   cRow += left( cNote, 30 )   
   
   // output the row:
   fOut.puts( cRow )

   rHE.next() // next row
enddo // end of loop

// cleanup:
fOut.close()
release object fOut
fOut := null
qHE.active := false
release object qHE
qHE := null
dData.active := false
release object dData
dData := null

// done!


function CenterText( nWidth, cText )
   // to center text we need to know the width of the
   // report, and the text we want to center:
   local nSpaces
   // subtract the length of the text to be centered
   // from the width we are centering IN, and
   // divide by 2:
   nSpaces = ( nWidth - len( cText ) ) / 2
   // add number of spaces on the left:
   cText = replicate( " ", nSpaces ) + cText
return cText